<?php
namespace Teknisa\CADFIS\Strategies;

use Teknisa\Libs\Util\Environment;
use Teknisa\CADFIS\Strategies\GeneralStrategy;
use Teknisa\CADFIS\Services\FornecedorService as FornecedorService;

class FornecedorStrategy extends GeneralStrategy implements Interfaces\IStrategy
{
    protected $service;
    protected $errors = array();
    
    public function __construct(FornecedorService $service, Environment $environment)
    {
		$this->service = $service;
		$this->environment = $environment;
	}
	
	public function validate(&$data)
    {
        foreach ($data as &$row) {
            parent::verifyIsEmpty($row, $row["ROWNUM"], ['IDTPIJURFORN', 'NRINSJURFORN', 'NMRAZSOCFORN', 'NMFANTFORN', 
                                                         'IDENDEMISFOR', 'IDENDCFOPFOR', 'IDPRIPUBFORN', 
                                                         'CDPAIS', 'SGESTADO', 'CDMUNICIPIO', 'DSENDEFORN',
                                                         'CDPAISENTR', 'SGESTADOENTR', 'CDMUNICIPIOENTR', 'DSENDEFORNENTR',
                                                         'CDPAISCOBR', 'SGESTADOCOBR', 'CDMUNICIPIOCOBR', 'DSENDEFORNCOBR']);
                                                         
            //Fornecedor
            parent::verifyIdTpIJurForn($row, $row["ROWNUM"]);
            parent::verifyNrInsJurForn($row, $row["ROWNUM"]);
            parent::concatCdForneced($row);
            self::verifyFornecedorExists($row, $row["ROWNUM"]);
            self::verifyFornecedorRepeat($row, $row["ROWNUM"]);
            self::verifyNrInsEstForn($row, $row["ROWNUM"]);
            self::verifySizeOnly($row, $row["ROWNUM"], 'NMRAZSOCFORN', 100);
            self::verifySizeOnly($row, $row["ROWNUM"], 'NMFANTFORN', 40);
            self::verifyIdEndEmisFor($row, $row["ROWNUM"]);
            self::verifyIdEndCfopFor($row, $row["ROWNUM"]);
            self::verifyIdPriPubForn($row, $row["ROWNUM"]);
            
            //Endereco principal
            parent::verifyCdPais($row, $row["ROWNUM"]);
            self::verifyCdPaisExists($row, $row["ROWNUM"]);
            parent::verifySgEstado($row, $row["ROWNUM"]);
            self::verifySgEstadoExists($row, $row["ROWNUM"]);
            parent::verifyCdMunicipio($row, $row["ROWNUM"]);
            self::verifyCdMunicipioExists($row, $row["ROWNUM"]);
            self::verifySizeOnly($row, $row["ROWNUM"], 'DSENDEFORN', 50);
            self::verifySizeOnly($row, $row["ROWNUM"], 'DSCOMPENDFOR', 200);
            self::verifySizeOnly($row, $row["ROWNUM"], 'NMBAIRFORN', 30);
            self::verifyNCepForn($row, $row["ROWNUM"]);
            self::verifySizeOnly($row, $row["ROWNUM"], 'DSEMAILFORN', 40);
            self::verifySizeOnly($row, $row["ROWNUM"], 'DSEMAILFORNNFE', 120);
            self::verifySizeOnly($row, $row["ROWNUM"], 'NRCELULARFOR', 25);
            self::verifySizeOnly($row, $row["ROWNUM"], 'NRTELRESIFOR', 25);
            
            //Endereco de entrega
            parent::verifyCdPais($row, $row["ROWNUM"], 'CDPAISENTR');
            self::verifyCdPaisExists($row, $row["ROWNUM"], 'CDPAISENTR');
            parent::verifySgEstado($row, $row["ROWNUM"], 'SGESTADOENTR');
            self::verifySgEstadoExists($row, $row["ROWNUM"], 'CDPAISENTR', 'SGESTADOENTR');
            parent::verifyCdMunicipio($row, $row["ROWNUM"], 'CDMUNICIPIOENTR');
            self::verifyCdMunicipioExists($row, $row["ROWNUM"], 'CDPAISENTR', 'SGESTADOENTR', 'CDMUNICIPIOENTR');
            self::verifySizeOnly($row, $row["ROWNUM"], 'DSENDEFORNENTR', 50);
            self::verifySizeOnly($row, $row["ROWNUM"], 'DSCOMPENDFORENTR', 200);
            self::verifySizeOnly($row, $row["ROWNUM"], 'NMBAIRFORNENTR', 30);
            self::verifyNCepForn($row, $row["ROWNUM"], 'NRCEPFORNENTR');
            self::verifySizeOnly($row, $row["ROWNUM"], 'DSEMAILFORNENTR', 40);
            self::verifySizeOnly($row, $row["ROWNUM"], 'DSEMAILFORNNFEENTR', 120);
            self::verifySizeOnly($row, $row["ROWNUM"], 'NRCELULARFORENTR', 25);
            self::verifySizeOnly($row, $row["ROWNUM"], 'NRTELRESIFORENTR', 25);
            
            //Endereco de cobranca
            parent::verifyCdPais($row, $row["ROWNUM"], 'CDPAISCOBR');
            self::verifyCdPaisExists($row, $row["ROWNUM"], 'CDPAISCOBR');
            parent::verifySgEstado($row, $row["ROWNUM"], 'SGESTADOCOBR');
            self::verifySgEstadoExists($row, $row["ROWNUM"], 'CDPAISCOBR', 'SGESTADOCOBR');
            parent::verifyCdMunicipio($row, $row["ROWNUM"], 'CDMUNICIPIOCOBR');
            self::verifyCdMunicipioExists($row, $row["ROWNUM"], 'CDPAISCOBR', 'SGESTADOCOBR', 'CDMUNICIPIOCOBR');
            self::verifySizeOnly($row, $row["ROWNUM"], 'DSENDEFORNCOBR', 50);
            self::verifySizeOnly($row, $row["ROWNUM"], 'DSCOMPENDFORCOBR', 200);
            self::verifySizeOnly($row, $row["ROWNUM"], 'NMBAIRFORNCOBR', 30);
            self::verifyNCepForn($row, $row["ROWNUM"], 'NRCEPFORNCOBR');
            self::verifySizeOnly($row, $row["ROWNUM"], 'DSEMAILFORNCOBR', 40);
            self::verifySizeOnly($row, $row["ROWNUM"], 'DSEMAILFORNNFECOBR', 120);
            self::verifySizeOnly($row, $row["ROWNUM"], 'NRCELULARFORCOBR', 25);
            self::verifySizeOnly($row, $row["ROWNUM"], 'NRTELRESIFORCOBR', 25);
            
            $row['ERROS'] = $this->errors;
            
            parent::clearErrors();
        }
    }
    
    private function verifyNCepForn($row, $rowNum, $columnName = 'NRCEPFORN') {
        if(!is_null($row[$columnName])) {
            if (!ctype_digit(strval($row[$columnName]))) {
                parent::setError($rowNum, $columnName, $row[$columnName], "ONLY_ACCEPTS_NUMBERS");
            }
            
            if (strlen($row[$columnName]) > 15) {
                parent::setError($rowNum, $columnName, $row[$columnName], "15_NUMBERS_EXECEDED");
            }
        } 
    }
    
    private function verifySizeOnly(&$row, $rowNum, $columnName, $size)
    {
        $msgError = strval($size)."_CHAR_EXECEDED";
        if(!is_null($row[$columnName])) {
            if (strlen($row[$columnName]) > $size) {
                self::setError($rowNum, $columnName, $row[$columnName], $msgError);
            }
        }
    }
    
    private function verifyCdPaisExists($row, $rowNum, $columnName = 'CDPAIS')
    {
        if(!is_null($row[$columnName])) {
            if (!$this->service->verifyCdPaisExists($row, $columnName)) {
                parent::setError($rowNum, $columnName, $row[$columnName], "UNREGISTERED");
            }
        }
    }
    
    private function verifySgEstadoExists($row, $rowNum, $columnNameCdPais = 'CDPAIS', $columnNameSgEstado = 'SGESTADO')
    {
        if(!is_null($row[$columnNameCdPais]) && !is_null($row[$columnNameSgEstado])) {
            if (!$this->service->verifySgEstadoExists($row, $columnNameCdPais, $columnNameSgEstado)) {
                parent::setError($rowNum, $columnNameSgEstado, $row[$columnNameSgEstado], "UNREGISTERED");
            }
        }
    }
    
    private function verifyCdMunicipioExists($row, $rowNum, $columnNameCdPais = 'CDPAIS', $columnNameSgEstado = 'SGESTADO', $columnNameCdMunicipio = 'CDMUNICIPIO')
    {
        if(!is_null($row[$columnNameCdPais]) && !is_null($row[$columnNameSgEstado]) && !is_null($row[$columnNameCdMunicipio])) {
            if (!$this->service->verifyCdMunicipioExists($row, $columnNameCdPais, $columnNameSgEstado, $columnNameCdMunicipio)) {
                parent::setError($rowNum, $columnNameCdMunicipio, $row[$columnNameCdMunicipio], "UNREGISTERED");
            }
        }
    }
    
    private function verifyIdPriPubForn($row, $rowNum) {
        $columnName = 'IDPRIPUBFORN';
        
        if(!is_null($row[$columnName])) {
            if (strlen($row[$columnName]) > 1) {
                self::setError($rowNum, $columnName, $row[$columnName], "1_CHAR_EXECEDED");
            }
    
            if (!strpos('_V_B_',$row[$columnName])) {
                self::setError($rowNum, $columnName, $row[$columnName], "ONLY_ACCEPTS_V_B");
            }
        } 
    }
    
    private function verifyIdEndCfopFor($row, $rowNum) {
        $columnName = 'IDENDCFOPFOR';
        
        if(!is_null($row[$columnName])) {
            if (strlen($row[$columnName]) > 1) {
                self::setError($rowNum, $columnName, $row[$columnName], "1_CHAR_EXECEDED");
            }
    
            if (!strpos('_P_C_E_',$row[$columnName])) {
                self::setError($rowNum, $columnName, $row[$columnName], "ONLY_ACCEPTS_P_C_E");
            }
        } 
    }
    
    private function verifyIdEndEmisFor($row, $rowNum) {
        $columnName = 'IDENDEMISFOR';
        
        if(!is_null($row[$columnName])) {
            if (strlen($row[$columnName]) > 1) {
                self::setError($rowNum, $columnName, $row[$columnName], "1_CHAR_EXECEDED");
            }
    
            if (!strpos('_P_C_E_',$row[$columnName])) {
                self::setError($rowNum, $columnName, $row[$columnName], "ONLY_ACCEPTS_P_C_E");
            }
        } 
    }
    
    private function verifyNrInsEstForn($row, $rowNum) {
        $columnName = 'NRINSESTFORN';
        
        if(!is_null($row[$columnName])) {
            if (!ctype_digit(strval($row[$columnName]))) {
                parent::setError($rowNum, $columnName, $row[$columnName], "ONLY_ACCEPTS_NUMBERS");
            }
            
            if (strlen($row[$columnName]) > 20) {
                parent::setError($rowNum, $columnName, $row[$columnName], "20_NUMBERS_EXECEDED");
            }
        } 
    }
    
    private function verifyFornecedorExists($row, $rowNum)
    {
        $columnName = 'NRINSJURFORN';
        
        if(!is_null($row[$columnName])) {
            if ($this->service->verifyFornecedorExists($row)) {
                parent::setError($rowNum, $columnName, $row[$columnName], "ALREADY_REGISTERED");
            }
        }
    }
    
    private function verifyFornecedorRepeat($row, $rowNum)
    {
        $columnName = 'NRINSJURFORN';

        if(!is_null($row[$columnName]) && isset($row["FORNECEDORINVALIDO"])){
            parent::setError($rowNum, $columnName, $row[$columnName], "REPEATED_RECORD");
        }
    }
    
    public function formatToView(&$data)
    {
        foreach ($data as &$row) {
           $row['IDERROR'] = !empty($row['ERROS']) ? 'S' : 'N';
        }
    }
    
    public function formatToSave(&$data)
    {
        foreach ($data as &$row) {
            $row['__is_new'] = true;
            $row['IDTPENDEFORN']        = 'P';
            $row['IDTPENDEFORNENTR']    = 'E';
            $row['IDTPENDEFORNCOBR']    = 'C';
            
            $row['NRORG']               = $this->environment->getNrorg();
            $row['DTINCLUSAO']          = NULL;
            $row['DTULTATU']            = NULL;
            $row['NRORGINCLUSAO']       = $this->environment->getNrorg();
            $row['CDOPERINCLUSAO']      = $this->environment->getCdOperador();
            $row['NRORGULTATU']         = $this->environment->getNrorg();
            $row['CDOPERULTATU']        = $this->environment->getCdOperador();
            $row['IDATIVO']             = 'S';
        } 
    }
    
    public function save($data)
    {
        self::validate($data);
        parent::verifyExistErros($data);
        
        
        
        $this->service->saveImport($data);
    }
}