<?php
namespace Teknisa\CADFIS\Strategies;

use Teknisa\Libs\Util\Environment;
use Teknisa\CADFIS\Strategies\GeneralStrategy;
use Teknisa\CADFIS\Services\ProdCompService as ProdCompService;

class ProdCompStrategy extends GeneralStrategy implements Interfaces\IStrategy
{
    protected $service;
    protected $errors = array();
    
    public function __construct(ProdCompService $service, Environment $environment)
    {
		$this->service = $service;
		$this->environment = $environment;
	}
	
    public function validate(&$data)
    {
        //parent::verifyIsEmpty($data);
        
        foreach ($data as &$row) {
            parent::verifyIsEmpty($row, $row["ROWNUM"], ['CDOPERADOR', 'CDARVPROD']);
            parent::verifyCdOperador($row, $row["ROWNUM"], 'CDOPERADOR');
            parent::verifyCdProduto($row, $row["ROWNUM"]);
            self::verifyProdCompExists($row, $row["ROWNUM"]);
            
            $row['ERROS'] = $this->errors;
            
            parent::clearErrors();
        }
    }
    
    private function verifyProdCompExists($row, $rowNum)
    {
        if ($this->service->verifyProdCompExists($row)) {
            parent::setError($rowNum, 'CDOPERADOR/CDARVPROD', $row['CDOPERADOR'].$row['CDARVPROD'], "ALREADY_REGISTERED");
        }
    }
    
    public function formatToView(&$data)
    {
        foreach ($data as &$row) {
           $row['CDOPERADOR'] = parent::formatColumn($row['CDOPERADOR'], 12);
           $row['IDERROR'] = !empty($row['ERROS']) ? 'S' : 'N';
        }
    }
    
    public function save($data)
    {
        self::validate($data);
        parent::verifyExistErros($data);
        $this->service->saveImport($data);
    }
}